/*++
*******************************************************************************
* Copyright (c) 1999-2000 Digi International Inc., All Rights Reserved				
* 																										
* This software contains proprietary and confidential information of Digi		
* International Inc.  By accepting transfer of this copy, Recipient agrees		
* to retain this software in confidence, to prevent disclosure to others,		
* and to make no use of this software other than that for which it was			
* delivered.  This is an unpublished copyrighted work of Digi International	
* Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly	
* prohibited.																						
* 																										
* Restricted Rights Legend																		
*																										
* Use, duplication, or disclosure by the Government is subject to					
* restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in			
* Technical Data and Computer Software clause at DFARS 252.227-7031 or			
* subparagraphs (c)(1) and (2) of the Commercial Computer Software -				
* Restricted Rights at 48 CFR 52.227-19, as applicable.								
*																										
* Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343				
*																										
*******************************************************************************

Module Name:

   AsynRsp.h

Abstract:


Author:
	Sandy Haapala

--*/
#ifndef ASYNRSP_H
#define ASYNRSP_H

//
// S/DXB Response structures
//

typedef struct sdxb_sync_response SDXB_SYNC_RSP, *PSDXB_SYNC_RSP;
typedef struct sdxb_bufsize_response SDXB_BUFSIZE_RSP, *PSDXB_BUFSIZE_RSP;
typedef struct sdxb_mstat_response SDXB_MSTAT_RSP, *PSDXB_MSTAT_RSP;
typedef struct sdxb_event_response SDXB_EVENT_RSP, *PSDXB_EVENT_RSP;
typedef struct sdxb_lerr_response SDXB_LINE_ERR_RSP, *PSDXB_LINE_ERR_RSP;
typedef struct sdxb_pcap_response SDXB_PORT_CAP_RSP, *PSDXB_PORT_CAP_RSP;
typedef struct sdxb_param_response SDXB_PORT_PARAM_RSP, *PSDXB_PORT_PARAM_RSP;
typedef struct sdxb_seq_response SDXB_SEQ_NUM_RSP, *PSDXB_SEQ_NUM_RSP;
typedef struct sdxb_screcv_response SDXB_SCRECV_RSP, *PSDXB_SCRECV_RSP;
typedef struct sdxb_proterr_response SDXB_PROTOCOL_ERR_RSP, *PSDXB_PROTOCOL_ERR_RSP;


VOID PortOnSdxbBufsize(IN PASYN_PORT_EXT pPortExt, IN ULONG	Insize, IN ULONG Outsize); 
VOID PortOnSdxbMstat(IN PASYN_PORT_EXT pPortExt, IN UCHAR MStat );
VOID PortOnSdxbEvent(IN PASYN_PORT_EXT pPortExt, IN USHORT Estat );
VOID PortOnSdxbLerr(IN PASYN_PORT_EXT pPortExt, IN struct sdxb_lerr_response * pLineErr );
VOID PortOnSdxbPcap(IN PASYN_PORT_EXT pPortExt, IN struct sdxb_pcap_response * pPcap );
VOID PortOnSdxbParam(IN PASYN_PORT_EXT pPortExt, IN struct sdxb_param_response * pParam );
VOID PortOnSdxbSeq(IN PASYN_PORT_EXT pPortExt, IN ULONG rin, IN ULONG tout );
VOID PortOnSdxbFrexcp(IN PASYN_PORT_EXT pPortExt, IN UCHAR flags );
VOID PortOnSdxbTxprint(IN PASYN_PORT_EXT pPortExt);
VOID PortOnSdxbMarkidle(IN PASYN_PORT_EXT pPortExt );
VOID PortOnSdxbProterr(IN PASYN_PORT_EXT pPortExt, IN USHORT offset, IN UCHAR badreq[]);




VOID PortOnSdxbParam(IN PASYN_PORT_EXT	pPortExt, IN struct sdxb_param_command * pParam );
VOID PortOnSdxbScrecv( IN PASYN_PORT_EXT pPortExt );
VOID PortOnSdxbSync( IN PASYN_PORT_EXT pPortExt, IN USHORT SeqNum );
VOID PortOnSdxbIsdn( IN PASYN_PORT_EXT pPortExt, IN PVOID msg );

#endif
